==> libremail-2.6.1.tar.bz2: libremail/src/mailrep.c <==
/*
    Fichier mailrep.c
    Auteur Bernard Chardonneau

    Logiciel libre, droits d'utilisation prciss en franais
    dans le fichier : licence-fr.txt

    Traductions des droits d'utilisation dans les fichiers :
    licence-de.txt , licence-en.txt , licence-es.txt ,
    licence-it.txt , licence-nl.txt , licence-pt.txt ,
    licence-eo.txt , licence-eo-utf.txt

    Droits d'utilisation galement sur la page web :
    http://libremail.tuxfamily.org/voir.php?page=droits


    Bibliothque de fonctions permettant d'avertir un expditeur
    que son mail a t refus.
    Le message  envoyer est mmoris dans un fichier de donnes.

    Cette bibliothque de fonctions est utilise par les commandes
    supbcc, suphtm et supgrosmail .
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include "mailrep.h"
#include "messages.h"
#include "smtp.h"
#include "szchemin.h"

#define  szlig    120  // longueur maximale des lignes du fichier de donnes
#define  maxmemo    5  // nombre maximal de lignes du fichier mmorises


/* variables globales au source */

char chem_mailrep [szchemin]; // chemin d'accs au fichier de donnes
FILE *fmailrep = NULL;        // descripteur du fichier de donnes
int  nbligmemo = 0;           // nombre de mots cls mmoriss
char mot_cle [maxmemo][20];   // liste des mots cls mmoriss
char chaine [maxmemo][szlig]; // et des chaines de caractres associes
long posmessage = 0;          // position dans le fichier du corps du message



/* si ce n'est dj fait, ouvre de fichier dcrivant le contenu d'un mail
    envoyer automatiquement, et mmorise certaines donnes qu'il contient

   si le nom de fichier ficmailrep n'est pas NULL, c'est ce fichier qui sera
   utilis, sinon le nom du fichier est fabriqu  partir du mot cl ncom */

int acces_mailrep (char *ficmailrep, char *ncom)
{
    static int testacces = 0; // mmorise si on a dj appel cette fonction
    char nom_mailrep [20];    // nom du fichier sans suffixe linguistique
    char ligne [szlig];       // contenu d'une ligne du fichier de donnes
    int  i, j;                // indices pour traitement chaines de caractres


    // si on appelle cette fonction pour la premire fois
    if (testacces == 0)
    {
        // on ne repassera plus dans ce bloc d'instructions
        testacces = 1;

        // gnrer le chemin d'accs au fichier du mail en rponse

        // si c'est un fichier spcifique
        if (ficmailrep)
            // recopier son nom
            strcpy (chem_mailrep, ficmailrep);
        // sinon
        else
        {
            // fabriquer le nom du fichier  partir du mot cl ncom
            sprintf (nom_mailrep, "mail-%s", ncom);
            strcpy (chem_mailrep, chemfichlang (nom_mailrep, 0));
        }

        // tenter de l'ouvrir en lecture
        fmailrep = fopen (chem_mailrep, "r");

        // si l'ouverture s'est bien passe
        if (fmailrep)
        {
            // tant qu'on n'est pas arriv sur le ligne !MESSAGE, lire une ligne
            while (fgets (ligne, szlig, fmailrep) && ! posmessage)
            {
                // suppression de \n en fin de ligne
                ligne [strlen (ligne) - 1] = '\0';

                // si cette ligne contient une chaine  mmoriser
                if ('A' <= *ligne && *ligne <= 'Z')
                {
                    //
                    if (nbligmemo >= maxmemo)
                    {
                   // "Dbordement : recompiler mailrep avec maxmemo plus grand"
                        affiche_err ("AUGM_MAXMEMO");
                        exit (-1);
                    }

                    // la mmoriser
                    i = 0;

                    // mmorisation du mot cl
                    while (ligne [i] > ' ')
                        mot_cle [nbligmemo][i++] = ligne [i];

                    mot_cle [nbligmemo][i++] = '\0';

                    // aller sur la chaine de caractres associe
                    while (ligne [i] == ' ' || ligne [i] == '\t')
                        i++;

                    j = 0;

                    // la mmorisation de la chaine diffre selon
                    // qu'elle soit encadre ou non de '"'
                    if (ligne [i] == '"')
                    {
                        while (ligne [++i] != '"' && i < szlig)
                        {
                            if (ligne [i] == '\\')
                                i++;

                            chaine [nbligmemo][j++] = ligne [i];
                        }
                    }
                    else
                    {
                        while (ligne [i] > ' ')
                            chaine [nbligmemo][j++] = ligne [i++];
                    }

                    // fin de la mmorisation de la chaine associe
                    chaine [nbligmemo++][j] = '\0';
                }

                // sinon, si on vient de lire la ligne !MESSAGE
                else if (*ligne == '!' && strcmp (ligne, "!MESSAGE") == 0)
                    // mmoriser la position du dbut du message dans le fichier
                    posmessage = ftell (fmailrep);
            }
        }
        // sinon, message d'erreur (qui ne sera pas rpt)
        else
        {
            // "Fichier %s manquant ou inaccessible"
            // "Les auteurs de mails dtruits ne seront pas avertis"
            aff_err_arg ("FICH_MANQUANT", chem_mailrep);
            affiche_err ("PAS_AVERTIS");
        }
    }

    // retourner le rsultat des oprations de chargement
    return ((int) fmailrep);
}



/* retourne une chaine de caractres identifie par un nom */

char *chaine_mailrep (char *motcle)
{
    int ligmemo;


    // chercher le mot cl dans la liste
    for (ligmemo = 0; ligmemo < nbligmemo; ligmemo++)
    {
        // si trouv, retourner la chaine de caractres associe
        if (strcmp (mot_cle [ligmemo], motcle) == 0)
            return (chaine [ligmemo]);
    }

    // sinon, message d'erreur fatale
    // "Erreur dans le fichier %s :"
    // "Mot cl %s manquant"
    aff_err_arg ("ERREUR_FICHIER", chem_mailrep);
    errfatale   ("MOT_CLE_ABSENT", motcle);
}



/* envoie le corps du mail */

void envoi_mailrep ()
{
    char ligne [szlig];  // contenu d'une ligne du fichier de donnes


    // se positionner au dbut du message
    fseek (fmailrep, posmessage, SEEK_SET);

    // tant qu'on n'a pas envoy tout le message, lire une ligne
    while (fgets (ligne, szlig, fmailrep))
    {
        // supprimer le \n en fin de ligne
        ligne [strlen (ligne) - 1] = '\0';

        // envoyer la ligne dans le mail
        envoie_smtp (ligne);
    }
}

==> libremail-2.7.tar.bz2: libremail/src/mailrep.c <==
/*
    Fichier mailrep.c
    Auteur Bernard Chardonneau

    Logiciel libre, droits d'utilisation prciss en franais
    dans le fichier : licence-fr.txt

    Traductions des droits d'utilisation dans les fichiers :
    licence-de.txt , licence-en.txt , licence-es.txt ,
    licence-it.txt , licence-nl.txt , licence-pt.txt ,
    licence-eo.txt , licence-eo-utf.txt

    Droits d'utilisation galement sur la page web :
    http://libremail.tuxfamily.org/voir.php?page=droits


    Bibliothque de fonctions permettant d'avertir un expditeur
    que son mail a t refus.
    Le message  envoyer est mmoris dans un fichier de donnes.

    Cette bibliothque de fonctions est utilise par les commandes
    supbcc, suphtm et supgrosmail .
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include "mailrep.h"
#include "messages.h"
#include "smtp.h"
#include "szchemin.h"

#define  szlig    120  // longueur maximale des lignes du fichier de donnes
#define  maxmemo    5  // nombre maximal de lignes du fichier mmorises


/* variables globales au source */

char chem_mailrep [szchemin]; // chemin d'accs au fichier de donnes
FILE *fmailrep = NULL;        // descripteur du fichier de donnes
int  nbligmemo = 0;           // nombre de mots cls mmoriss
char mot_cle [maxmemo][20];   // liste des mots cls mmoriss
char chaine [maxmemo][szlig]; // et des chaines de caractres associes
long posmessage = 0;          // position dans le fichier du corps du message



/* si ce n'est dj fait, ouvre de fichier dcrivant le contenu d'un mail
    envoyer automatiquement, et mmorise certaines donnes qu'il contient

   si le nom de fichier ficmailrep n'est pas NULL, c'est ce fichier qui sera
   utilis, sinon le nom du fichier est fabriqu  partir du mot cl ncom */

FILE * acces_mailrep (char *ficmailrep, char *ncom)
{
    static int testacces = 0; // mmorise si on a dj appel cette fonction
    char nom_mailrep [20];    // nom du fichier sans suffixe linguistique
    char ligne [szlig];       // contenu d'une ligne du fichier de donnes
    int  i, j;                // indices pour traitement chaines de caractres


    // si on appelle cette fonction pour la premire fois
    if (testacces == 0)
    {
        // on ne repassera plus dans ce bloc d'instructions
        testacces = 1;

        // gnrer le chemin d'accs au fichier du mail en rponse

        // si c'est un fichier spcifique
        if (ficmailrep)
            // recopier son nom
            strcpy (chem_mailrep, ficmailrep);
        // sinon
        else
        {
            // fabriquer le nom du fichier  partir du mot cl ncom
            sprintf (nom_mailrep, "mail-%s", ncom);
            strcpy (chem_mailrep, chemfichlang (nom_mailrep, 0));
        }

        // tenter de l'ouvrir en lecture
        fmailrep = fopen (chem_mailrep, "r");

        // si l'ouverture s'est bien passe
        if (fmailrep)
        {
            // tant qu'on n'est pas arriv sur le ligne !MESSAGE, lire une ligne
            while (fgets (ligne, szlig, fmailrep) && ! posmessage)
            {
                // suppression de \n en fin de ligne
                ligne [strlen (ligne) - 1] = '\0';

                // si cette ligne contient une chaine  mmoriser
                if ('A' <= *ligne && *ligne <= 'Z')
                {
                    //
                    if (nbligmemo >= maxmemo)
                    {
                   // "Dbordement : recompiler mailrep avec maxmemo plus grand"
                        affiche_err ("AUGM_MAXMEMO");
                        exit (-1);
                    }

                    // la mmoriser
                    i = 0;

                    // mmorisation du mot cl
                    while (ligne [i] > ' ')
                        mot_cle [nbligmemo][i++] = ligne [i];

                    mot_cle [nbligmemo][i++] = '\0';

                    // aller sur la chaine de caractres associe
                    while (ligne [i] == ' ' || ligne [i] == '\t')
                        i++;

                    j = 0;

                    // la mmorisation de la chaine diffre selon
                    // qu'elle soit encadre ou non de '"'
                    if (ligne [i] == '"')
                    {
                        while (ligne [++i] != '"' && i < szlig)
                        {
                            if (ligne [i] == '\\')
                                i++;

                            chaine [nbligmemo][j++] = ligne [i];
                        }
                    }
                    else
                    {
                        while (ligne [i] > ' ')
                            chaine [nbligmemo][j++] = ligne [i++];
                    }

                    // fin de la mmorisation de la chaine associe
                    chaine [nbligmemo++][j] = '\0';
                }

                // sinon, si on vient de lire la ligne !MESSAGE
                else if (*ligne == '!' && strcmp (ligne, "!MESSAGE") == 0)
                    // mmoriser la position du dbut du message dans le fichier
                    posmessage = ftell (fmailrep);
            }
        }
        // sinon, message d'erreur (qui ne sera pas rpt)
        else
        {
            // "Fichier %s manquant ou inaccessible"
            // "Les auteurs de mails dtruits ne seront pas avertis"
            aff_err_arg ("FICH_MANQUANT", chem_mailrep);
            affiche_err ("PAS_AVERTIS");
        }
    }

    // retourner le rsultat des oprations de chargement
    return (fmailrep);
}



/* retourne une chaine de caractres identifie par un nom */

char *chaine_mailrep (char *motcle)
{
    int ligmemo;


    // chercher le mot cl dans la liste
    for (ligmemo = 0; ligmemo < nbligmemo; ligmemo++)
    {
        // si trouv, retourner la chaine de caractres associe
        if (strcmp (mot_cle [ligmemo], motcle) == 0)
            return (chaine [ligmemo]);
    }

    // sinon, message d'erreur fatale
    // "Erreur dans le fichier %s :"
    // "Mot cl %s manquant"
    aff_err_arg ("ERREUR_FICHIER", chem_mailrep);
    errfatale   ("MOT_CLE_ABSENT", motcle);
}



/* envoie le corps du mail */

void envoi_mailrep ()
{
    char ligne [szlig];  // contenu d'une ligne du fichier de donnes


    // se positionner au dbut du message
    fseek (fmailrep, posmessage, SEEK_SET);

    // tant qu'on n'a pas envoy tout le message, lire une ligne
    while (fgets (ligne, szlig, fmailrep))
    {
        // supprimer le \n en fin de ligne
        ligne [strlen (ligne) - 1] = '\0';

        // envoyer la ligne dans le mail
        envoie_smtp (ligne);
    }
}
